@@ -84,7 +84,7 @@ class Module:
             try:
                 os.makedirs(user_autostart_dir)
             except:
-                print("Could not create autostart dir: %s" % user_autostart_dir)
+                print(f"Could not create autostart dir: {user_autostart_dir}")
 
     def gather_apps(self):
         system_files = []
@@ -129,7 +129,7 @@ class AutostartApp:
         try:
             self.key_file.load_from_file(self.app, KEYFILE_FLAGS)
         except GLib.GError as e:
-            print("Failed to load %s" % self.app, e)
+            print(f"Failed to load {self.app}", e)
             return
 
         self.key_file_loaded = True
@@ -227,7 +227,7 @@ class AutostartApp:
             else:
                 key_file.load_from_file(self.path, KEYFILE_FLAGS)
         except Exception as e:
-            print("Problem creating user keyfile: %s" % e)
+            print(f"Problem creating user keyfile: {e}")
             key_file.set_string(D_GROUP, GLib.KEY_FILE_DESKTOP_KEY_TYPE, "Application")
             key_file.set_string(D_GROUP, GLib.KEY_FILE_DESKTOP_KEY_EXEC, "/bin/false")
 
@@ -421,27 +421,27 @@ class AutostartBox(Gtk.Box):
         box.set_halign(Gtk.Align.CENTER)
         button_holder.add(box)
 
-        self.add_button = Gtk.Button.new_from_icon_name("list-add-symbolic", Gtk.IconSize.BUTTON)
+        self.add_button = Gtk.Button.new_from_icon_name("xsi-list-add-symbolic", Gtk.IconSize.BUTTON)
         self.add_button.set_tooltip_text(_("Add"))
         self.add_button.connect("clicked", self.on_add_button_clicked)
         button_group.add_widget(self.add_button)
         box.add(self.add_button)
 
-        self.edit_button = Gtk.Button.new_from_icon_name("document-edit-symbolic", Gtk.IconSize.BUTTON)
+        self.edit_button = Gtk.Button.new_from_icon_name("xsi-document-edit-symbolic", Gtk.IconSize.BUTTON)
         self.edit_button.set_tooltip_text(_("Edit"))
         self.edit_button.connect("clicked", self.on_edit_button_clicked)
         button_group.add_widget(self.edit_button)
         self.edit_button.set_sensitive(False)
         box.add(self.edit_button)
 
-        self.remove_button = Gtk.Button.new_from_icon_name("list-remove-symbolic", Gtk.IconSize.BUTTON)
+        self.remove_button = Gtk.Button.new_from_icon_name("xsi-list-remove-symbolic", Gtk.IconSize.BUTTON)
         self.remove_button.set_tooltip_text(_("Remove"))
         self.remove_button.connect("clicked", self.on_remove_button_clicked)
         button_group.add_widget(self.remove_button)
         self.remove_button.set_sensitive(False)
         box.add(self.remove_button)
 
-        self.run_button = Gtk.Button.new_from_icon_name("media-playback-start-symbolic", Gtk.IconSize.BUTTON)
+        self.run_button = Gtk.Button.new_from_icon_name("xsi-media-playback-start-symbolic", Gtk.IconSize.BUTTON)
         self.run_button.set_tooltip_text(_("Run now"))
         self.run_button.connect("clicked", self.on_run_button_clicked)
         button_group.add_widget(self.run_button)
@@ -584,7 +584,7 @@ class AutostartBox(Gtk.Box):
             try:
                 shutil.copyfile(desktop_file_dir, user_desktop_file)
             except IOError:
-                print("Failed to copy desktop file %s" % desktop_file_name)
+                print(f"Failed to copy desktop file {desktop_file_name}")
 
             app = AutostartApp(user_desktop_file, user_position=os.path.dirname(user_desktop_file))
             key = get_appname(user_desktop_file)
@@ -622,14 +622,14 @@ class AutostartBox(Gtk.Box):
         else:
             base_path = os.path.join(GLib.get_user_config_dir(), "autostart", suggested_name)
 
-        filename = "%s.desktop" % base_path
+        filename = f"{base_path}.desktop"
         basename = os.path.basename(filename)
 
         i = 1
         max_tries = 100
         while (self.find_app_with_basename(basename) is not None and
                i < max_tries):
-            filename = "%s-%d.desktop" % (base_path, i)
+            filename = f"{base_path}-{i}.desktop"
             basename = os.path.basename(filename)
             i += 1
 
@@ -863,7 +863,7 @@ class AppDialog(Gtk.Dialog):
             if error_msg is not None:
                 msg_box = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR,
                                             Gtk.ButtonsType.CANCEL,
-                                            "%s" % error_msg)
+                                            error_msg)
                 error_msg = None
                 msg_box.run()
                 msg_box.destroy()
@@ -899,6 +899,8 @@ class AppDialog(Gtk.Dialog):
 
         if response == Gtk.ResponseType.ACCEPT:
             name = chooser.get_filename()
+            if " " in name:
+                name = '"' + name + '"'
             self.command_entry.set_text(name)
 
         chooser.destroy()
